//
//  ViewController.swift
//  MassageMe
//
//  Created by Megan Leszczynski on 11/18/16.
//  Copyright © 2016 Harry+Margi. All rights reserved.
//

import UIKit
import CoreBluetooth
import ChameleonFramework
import KRProgressHUD

class ViewController: UIViewController, BluetoothSerialDelegate  {

    var recvbuffer = ""
    var imageView = UIImageView(image: UIImage(named: "vestoutline2.png")!)
    let screenSize: CGRect = UIScreen.mainScreen().bounds
    let label = UILabel(frame: CGRect(x: 0, y: 160, width: 200, height: 50))
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.
        serial = BluetoothSerial(delegate: self)
        serial.writeType = .WithoutResponse
        
        if serial.state == .PoweredOff {
            print("Bluetooth not turned on")
            return
        }
        self.view.backgroundColor = UIColor.blackColor()
        
        KRProgressHUD.show(progressHUDStyle: KRProgressHUDStyle.BlackColor)
        KRProgressHUD.setDefaultCenterPosition(CGPoint(x: self.view.center.x, y: self.view.center.y))
        imageView.frame = CGRectMake(0,0,200,250)
        imageView.center.x = self.view.center.x
        imageView.center.y = self.view.center.y
        self.view.addSubview(imageView)
        
        label.backgroundColor = UIColor.blackColor()
        label.textColor = UIColor.whiteColor()
        label.center.x = self.view.center.x
        label.textAlignment = NSTextAlignment.Center
        label.text = "MASSAGEME"
        label.font = UIFont(name: "ArialUnicodeMS", size: 30)
        label.alpha = 0
        self.view.addSubview(label)

        self.navigationController?.navigationBar.setBackgroundImage(UIImage(), forBarMetrics: .Default)
        self.navigationController?.navigationBar.shadowImage = UIImage()
        self.navigationController?.navigationBar.translucent = true
        self.navigationController?.navigationBar.tintColor = UIColor.whiteColor()
        
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    override func viewDidAppear(animated: Bool) {
        serial.sendBytesToDevice([stop_msg, eot])
    }
    
    //MARK: BluetoothSerialDelegate
        
    func serialDidDisconnect(peripheral: CBPeripheral, error: NSError?) {
        print("Disconnect.")
    }
    
    func serialDidChangeState(newState: CBCentralManagerState) {
        let delay = 3.0 * Double(NSEC_PER_SEC)
        let time = dispatch_time(DISPATCH_TIME_NOW, Int64(delay))
        dispatch_after(time, dispatch_get_main_queue()) {
            if newState == .PoweredOn {
                serial.startScan()
            }
        }
        print("Failed to connect.")
    }

    func serialDidDiscoverPeripheral(peripheral: CBPeripheral, RSSI: NSNumber?) {
        if peripheral.name == "MASSAGEME" {
            serial.stopScan()
            // can't scan and connect at same time
            serial.connectToPeripheral(peripheral)
        }
    }
    
    func serialDidFailToConnect(peripheral: CBPeripheral, error: NSError?) {
        print("Failed to connect.")
    }
    
    func serialIsReady(peripheral: CBPeripheral) {
        print("Connected successfully.")
        KRProgressHUD.dismiss()
        UIView.animateWithDuration(0.75, delay: 0.25,
                                        options: .CurveLinear,
                                        animations: {self.imageView.center.y -= 100},
                                        completion: nil)
        
        UIView.animateWithDuration(0.5, delay: 1.0,
                                   options: .TransitionCrossDissolve,
                                   animations: {self.label.alpha = 1.0},
                                   completion: nil)
        
        // buttons
        let presetButton = UIButton(frame: CGRect(x: 100, y: 375, width: 250, height: 50))
        presetButton.backgroundColor = UIColor.blackColor()
        presetButton.setTitleColor(HexColor("8b008B"), forState: UIControlState.Normal)
        presetButton.layer.cornerRadius = 25
        presetButton.layer.borderColor = HexColor("8b008B").CGColor
        presetButton.layer.borderWidth = 1;
        presetButton.clipsToBounds = true
        presetButton.setTitle("PRESET", forState: UIControlState.Normal)
        presetButton.addTarget(self, action: #selector(presetPushed), forControlEvents: .TouchUpInside)
        presetButton.center.x = self.view.center.x
        presetButton.alpha = 0.0
        self.view.addSubview(presetButton)
        
        let customButton = UIButton(frame: CGRect(x: 100, y: presetButton.center.y + 35, width: 250, height: 50))
        customButton.backgroundColor = UIColor.blackColor()
        customButton.setTitleColor(HexColor("8b008B"), forState: UIControlState.Normal)
        customButton.layer.cornerRadius = 25
        customButton.layer.borderColor = HexColor("8b008B").CGColor
        customButton.layer.borderWidth = 1;
        customButton.clipsToBounds = true
        customButton.setTitle("CUSTOM", forState: UIControlState.Normal)
        customButton.addTarget(self, action: #selector(customPushed), forControlEvents: .TouchUpInside)
        customButton.center.x = self.view.center.x
        customButton.alpha = 0.0
        self.view.addSubview(customButton)
        
        let drawButton = UIButton(frame: CGRect(x: 100, y: customButton.center.y + 35, width: 250, height: 50))
        drawButton.backgroundColor = UIColor.blackColor()
        drawButton.setTitleColor(HexColor("8b008B"), forState: UIControlState.Normal)
        drawButton.layer.cornerRadius = 25
        drawButton.layer.borderColor = HexColor("8b008B").CGColor
        drawButton.layer.borderWidth = 1;
        drawButton.clipsToBounds = true
        drawButton.setTitle("DRAW", forState: UIControlState.Normal)
        drawButton.addTarget(self, action: #selector(drawPushed), forControlEvents: .TouchUpInside)
        drawButton.center.x = self.view.center.x
        drawButton.alpha = 0.0
        self.view.addSubview(drawButton)
        
        UIView.animateWithDuration(0.5, delay: 0.5,
                                        options: .TransitionCrossDissolve,
                                        animations: {presetButton.alpha = 1.0; customButton.alpha = 1.0; drawButton.alpha = 1.0},
                                        completion: nil)
    }

    func buttonAction(sender: UIButton!) {
        print("Button tapped")
        if serial.isReady{
            print("Sending data")
            //var msg = "\r\n"
            //let msg = "gargi sends her love"
            //serial.sendMessageToDevice(msg)
            serial.sendBytesToDevice([54,50,50,50,13])
        }
    }
    
    func presetPushed(sender: UIButton!) {
        self.navigationController?.pushViewController(PresetController(), animated: true)
    }
    
    func customPushed(sender: UIButton!) {
        self.navigationController?.pushViewController(CustomController(), animated: true)
    }
    
    func drawPushed(sender: UIButton!) {
        self.navigationController?.pushViewController(DrawController(), animated: true)
    }
    
    
}

