/* 
 * finalproject.c
 * 
 * 11.30.16
 * Harry Freeman (hnf22)
 * Gargi Ratnaparkhi (gpr29) 
 * Megan Leszczynski (mel255)  
*/

#include "config.h"
#include <stdlib.h>
#include <math.h>
#include <plib.h>
#include "pt_cornell_1_2_1.h.h"
#include "port_expander.h"

// Port B
#define EnablePullDownB(bits) CNPUBCLR=bits; CNPDBSET=bits;
#define DisablePullDownB(bits) CNPDBCLR=bits;
#define EnablePullUpB(bits) CNPDBCLR=bits; CNPUBSET=bits;
#define DisablePullUpB(bits) CNPUBCLR=bits;
// Port A
#define EnablePullDownA(bits) CNPUACLR=bits; CNPDASET=bits;
#define DisablePullDownA(bits) CNPDACLR=bits;
#define EnablePullUpA(bits) CNPDACLR=bits; CNPUASET=bits;
#define DisablePullUpA(bits) CNPUACLR=bits;

// Fix16
typedef signed int fix16 ;
#define multfix16(a,b) ((fix16)(((( signed long long)(a))*(( signed long long)(b)))>>16)) //multiply two fixed 16:16
#define float2fix16(a) ((fix16)((a)*65536.0)) // 2^16
#define fix2float16(a) ((float)(a)/65536.0)
#define fix2int16(a)    ((int)((a)>>16))
#define int2fix16(a)    ((fix16)((a)<<16))
#define divfix16(a,b) ((fix16)((((signed long long)(a)<<16)/(b)))) 
#define sqrtfix16(a) (float2fix16(sqrt(fix2float16(a)))) 
#define absfix16(a) abs(a)
#define onefix16 0x00010000 // int2fix16(1)
#define sustain_constant float2fix16(256.0/20000.0) ; // seconds per decay update

// Protocol
#define PRESET_MSG 1
#define CUSTOM_MSG 2
#define DRAW_MSG 3
#define STOP_MSG 0

#define FULL_BACK 0
#define UPPER_BACK 1
#define LOWER_BACK 2
#define SPINE 3

#define UPPER_LEFT_IDX 1
#define UPPER_RIGHT_IDX 2
#define LOWER_LEFT_IDX 3
#define LOWER_RIGHT_IDX 4

#define OFF 0 
#define LOW 1
#define MEDIUM 2
#define HIGH 3 

#define EOT 127
#define RESET 5 

// A
#define UPPER_LEFT_BITS  BIT_4|BIT_2|BIT_1|BIT_0 

// B 
#define UPPER_RIGHT_BITS BIT_4|BIT_3|BIT_1

// Y 
#define LOWER_LEFT_BITS BIT_7|BIT_6|BIT_5|BIT_4

// Z
#define LOWER_RIGHT_BITS BIT_3|BIT_2|BIT_1|BIT_0 

#define PWM_HIGH 15000
#define PWM_MED  10000
#define PWM_LOW  5000

#define B0 4
#define B0_BIT BIT_3 // B0 changed to correspond to Y3

static unsigned int pattern_A;
static unsigned int pattern_B;
static unsigned char out_Y;
static unsigned char out_Z;

char buffer[60]; // String buffer
static struct pt pt_bluetooth, pt_preset, pt_custom, pt_draw, pt_DMA_output, pt_input, pt_reply;
static struct pt *running_pt; 
static float value;

static char custom_flag, preset_flag, draw_flag; 
static char *running_massage = NULL; 

static char bluetooth_msg[max_chars];

//PWM
//The actual period of the wave
int generate_period = 40000;
int pwm_on_time = 20000;

void clear_buzzers(){
    mPORTAClearBits(UPPER_LEFT_BITS); // clear bits
    mPORTBClearBits(UPPER_RIGHT_BITS); // clear bits 
    writePE(GPIOY, 0); // clear bits  
    writePE(GPIOZ, 0); // clear bits  
    SetDCOC1PWM(0); // turn off other PWMs
    SetDCOC2PWM(0); 
    SetDCOC3PWM(0); 
    SetDCOC4PWM(0); 
}

//void update_pwm_b(int strength){
//    if (strength == LOW) {
//        pwm_on_time = PWM_LOW_B;
//    }
//    else if (strength == MEDIUM){
//        pwm_on_time = PWM_MED_B;
//    }
//    else if (strength == HIGH) {
//        pwm_on_time = PWM_HIGH_B; 
//    }
//}

void update_pwm(int strength){
    if (strength == LOW) {
        pwm_on_time = PWM_LOW;
    }
    else if (strength == MEDIUM){
        pwm_on_time = PWM_MED;
    }
    else if (strength == HIGH) {
        pwm_on_time = PWM_HIGH; 
    }
}

static char cycle_low = 0; // flip between settings to generate pattern
static PT_THREAD(protothread_preset(struct pt *pt)) { 
    int massage_type;
    PT_BEGIN(pt);
    while(1) {
        while(preset_flag) {
            clear_buzzers(); 
            massage_type = (int) bluetooth_msg[1];
            switch (massage_type){
                case FULL_BACK:
                    // upper back has more power
                    if (cycle_low) {
                        mPORTASetBits(UPPER_LEFT_BITS);
                        mPORTBSetBits(UPPER_RIGHT_BITS);
                        writePE(GPIOY, LOWER_LEFT_BITS | B0_BIT);
                        writePE(GPIOZ, LOWER_RIGHT_BITS);
                        SetDCOC1PWM(PWM_HIGH);
                        SetDCOC2PWM(PWM_HIGH);
                        SetDCOC3PWM(PWM_LOW);
                        SetDCOC4PWM(PWM_LOW);
                        cycle_low = 0;
                    }
                    else {
                        mPORTASetBits(UPPER_LEFT_BITS);
                        mPORTBSetBits(UPPER_RIGHT_BITS);
                        writePE(GPIOY, LOWER_LEFT_BITS | B0_BIT);
                        writePE(GPIOZ, LOWER_RIGHT_BITS);
                        SetDCOC1PWM(PWM_LOW);
                        SetDCOC2PWM(PWM_LOW);
                        SetDCOC3PWM(PWM_HIGH);
                        SetDCOC4PWM(PWM_HIGH);
                        cycle_low = 1;
                    }
                    break; 
                case UPPER_BACK:
                    writePE(GPIOY, 0); // clear lower bits  
                    writePE(GPIOZ, 0); // clear lower bits  
                    if (cycle_low) {
                        mPORTASetBits(UPPER_LEFT_BITS);
                        mPORTBSetBits(UPPER_RIGHT_BITS);
                        writePE(GPIOY, B0_BIT);
                        SetDCOC3PWM(PWM_LOW);
                        SetDCOC4PWM(PWM_HIGH);
                        cycle_low = 0; 
                    }
                    else {
                        mPORTASetBits(UPPER_LEFT_BITS);
                        mPORTBSetBits(UPPER_RIGHT_BITS);
                        writePE(GPIOY, B0_BIT);
                        SetDCOC3PWM(PWM_HIGH);
                        SetDCOC4PWM(PWM_LOW);
                        cycle_low = 1; 
                    }
                    break; 
                case LOWER_BACK:
                    mPORTAClearBits(UPPER_LEFT_BITS);
                    mPORTBClearBits(UPPER_RIGHT_BITS);
                    if (cycle_low) {
                        writePE(GPIOY, LOWER_LEFT_BITS);
                        writePE(GPIOZ, LOWER_RIGHT_BITS);
                        SetDCOC1PWM(PWM_LOW);
                        SetDCOC2PWM(PWM_HIGH);
                        cycle_low = 0; 
                    }
                    else {
                        writePE(GPIOY, LOWER_LEFT_BITS);
                        writePE(GPIOZ, LOWER_RIGHT_BITS);
                        SetDCOC1PWM(PWM_HIGH);
                        SetDCOC2PWM(PWM_LOW);
                        cycle_low = 1; 
                    }
                case SPINE:
                    clear_buzzers();
                    if (cycle_low) {
                        mPORTASetBits(BIT_1 | BIT_4);
                        mPORTBSetBits(BIT_3);
                        writePE(GPIOY, B0_BIT | BIT_5 | BIT_7);
                        writePE(GPIOZ, BIT_0 | BIT_2);
                        SetDCOC1PWM(PWM_HIGH);
                        SetDCOC2PWM(PWM_LOW);
                        SetDCOC3PWM(PWM_HIGH);
                        SetDCOC4PWM(PWM_LOW);
                        cycle_low = 0;
                    }
                    else {
                        mPORTASetBits(BIT_1 | BIT_4);
                        mPORTBSetBits(BIT_3);
                        writePE(GPIOY, B0_BIT | BIT_5 | BIT_7); // B0 on code moved to Y3
                        writePE(GPIOZ, BIT_0 | BIT_2);
                        SetDCOC1PWM(PWM_LOW);
                        SetDCOC2PWM(PWM_HIGH);
                        SetDCOC3PWM(PWM_LOW);
                        SetDCOC4PWM(PWM_HIGH);
                        cycle_low = 1;
                    }
                    break;    
            }
            PT_YIELD_TIME_msec(2000);
        }
        PT_YIELD(pt); 
    }
    PT_END(pt);
}

static PT_THREAD(protothread_custom(struct pt *pt)) {
    PT_BEGIN(pt);
    while(1){
        if (custom_flag){
            int strength;
            int b_on = 0; 
            clear_buzzers();

            // settings for upper left 
            strength = (int) bluetooth_msg[UPPER_LEFT_IDX];
            if (strength != OFF){
                mPORTASetBits(UPPER_LEFT_BITS);
                update_pwm(strength);
                SetDCOC3PWM(pwm_on_time);
            }

            // settings for upper right 
            strength = (int) bluetooth_msg[UPPER_RIGHT_IDX];
            if (strength != OFF){
                b_on = 1;
                mPORTBSetBits(UPPER_RIGHT_BITS);
                writePE(GPIOY, B0_BIT);
                update_pwm(strength);
                SetDCOC4PWM(pwm_on_time);
            }

            // settings for lower left 
            strength = (int) bluetooth_msg[LOWER_LEFT_IDX];
            if (strength != OFF){
                if (b_on){
                    writePE(GPIOY, LOWER_LEFT_BITS | B0_BIT);
                }
                else{
                    writePE(GPIOY, LOWER_LEFT_BITS);
                }
                update_pwm(strength);
                SetDCOC1PWM(pwm_on_time);
            }

            // settings for lower right 
            strength = (int) bluetooth_msg[LOWER_RIGHT_IDX];
            if (strength != OFF){
                writePE(GPIOZ, LOWER_RIGHT_BITS);
                update_pwm(strength);
                SetDCOC2PWM(pwm_on_time);
            }
            
            custom_flag = 0; // completed updating new custom settings 
        }
        PT_YIELD(pt);
    }
    PT_END(pt);
}

// mapping corresponds in clockwise direction 
// | 0 1 | -> | BIT_0 BIT_1 |
// | 2 3 |    | BIT_2 BIT_4 |
static int A_bits[4] = {BIT_0, BIT_1, BIT_2, BIT_4}; // spine:  1, 4
static int B_bits[4] = {BIT_0, BIT_1, BIT_3, BIT_4}; //         0, 3
static int Y_bits[4] = {BIT_4, BIT_5, BIT_6, BIT_7}; //         5, 7
static int Z_bits[4] = {BIT_0, BIT_1, BIT_2, BIT_3}; //         0, 2

// offsets of groups within the array of 16 buzzers 
#define A_OFFSET 0
#define B_OFFSET 4
#define Y_OFFSET 8
#define Z_OFFSET 12 

static int next_buzzer; 
static int msg_idx; 
static char stop_msg; 

static PT_THREAD(protothread_draw(struct pt *pt)) {
    PT_BEGIN(pt);
    
    while(1){
        if (draw_flag){
            msg_idx = 1; 
            next_buzzer = (int)bluetooth_msg[msg_idx];
    
            // set to full power for all regions - not decided by user 
            pwm_on_time = PWM_HIGH; 

            while(next_buzzer != EOT && draw_flag) {
                clear_buzzers();
                // turn on buzzer for 10 seconds 
                // A group (upper left)
                if (next_buzzer < B_OFFSET){
                    int bit = A_bits[next_buzzer];
                    mPORTASetBits(A_bits[next_buzzer]);
                    SetDCOC3PWM(pwm_on_time);
                }
                // B group (upper right)
                else if (next_buzzer < Y_OFFSET){
                    if (next_buzzer == B0){
                        writePE(GPIOY, B0_BIT); // B0 
                    }
                    else{
                        mPORTBSetBits(B_bits[next_buzzer-B_OFFSET]);
                    }
                    SetDCOC4PWM(pwm_on_time);
                }
                // Y group (lower left)
                else if (next_buzzer < Z_OFFSET){
                    writePE(GPIOY, Y_bits[next_buzzer-Y_OFFSET]);
                    SetDCOC1PWM(pwm_on_time);
                }
                // Z group (lower right)
                else {
                    writePE(GPIOZ, Z_bits[next_buzzer-Z_OFFSET]);
                    SetDCOC2PWM(pwm_on_time);
                }
                msg_idx++; 
                next_buzzer = (int) bluetooth_msg[msg_idx];
                PT_YIELD_TIME_msec(1000); // delay 1 sec before going to next buzzer
                // send message back
            }
            draw_flag = 0; // turn off draw flag after completion 
            clear_buzzers(); 
            running_massage = NULL; 
            PT_send_buffer[0] = RESET; 
            PT_send_buffer[1] = EOT;
            PT_SPAWN(pt, &pt_reply, PutSerialBuffer(&pt_reply));
        }
        PT_YIELD(pt); 
    }
    // send message to pic to indicate done 
    PT_END(pt);
}

static PT_THREAD (protothread_bluetooth (struct pt *pt)){
    PT_BEGIN(pt);
    while(1) {
            PT_SPAWN(pt, &pt_input, PT_GetSerialBuffer(&pt_input) );
            memcpy(bluetooth_msg, PT_term_buffer, max_chars);
            int msg_type = (int) bluetooth_msg[0];
            switch(msg_type){
                case STOP_MSG:
                    if (running_massage != NULL){
                        *running_massage = 0; // turn off flag
                        running_massage = NULL;
                        clear_buzzers();
                        stop_msg = 1; 
                    }
                    break;
                case PRESET_MSG:
                    preset_flag= 1; 
                    running_massage = &preset_flag; 
                    break;
                case CUSTOM_MSG:
                    custom_flag = 1; 
                    running_massage = &custom_flag;
                    break;
                case DRAW_MSG:
                    draw_flag = 1; 
                    running_massage = &draw_flag; 
                    break;
            }
    }
    PT_END(pt);
}

void main(void) {
    //SYSTEMConfigPerformance(PBCLK);
    ANSELA = 0; ANSELB = 0;
    
    PT_setup();
    INTEnableSystemMultiVectoredInt();
    
    OpenTimer2(T2_ON | T2_SOURCE_INT | T2_PS_1_1, 625);
    ConfigIntTimer2(T2_INT_OFF);
    //ConfigIntTimer2(T2_INT_ON | T2_INT_PRIOR_2);
    mT2ClearIntFlag(); // and clear the interrupt flag
      
    // PWM 
    OpenTimer3(T3_ON | T3_SOURCE_INT | T3_PS_1_1, generate_period);
    
    // Y 
    OpenOC1(OC_ON | OC_TIMER3_SRC | OC_PWM_FAULT_PIN_DISABLE , pwm_on_time, pwm_on_time);
    PPSOutput(1, RPB7, OC1);
    
    // Z
    OpenOC2(OC_ON | OC_TIMER3_SRC | OC_PWM_FAULT_PIN_DISABLE , pwm_on_time, pwm_on_time);
    PPSOutput(2, RPB8, OC2);
    
    // A
    OpenOC3(OC_ON | OC_TIMER3_SRC | OC_PWM_FAULT_PIN_DISABLE , pwm_on_time, pwm_on_time);
    PPSOutput(4, RPB14, OC3);
    
    // B
    OpenOC4(OC_ON | OC_TIMER3_SRC | OC_PWM_FAULT_PIN_DISABLE , pwm_on_time, pwm_on_time);
    PPSOutput(3, RPB13, OC4);

          // init the threads
    initPE();
    
    PT_INIT(&pt_bluetooth);
    PT_INIT(&pt_preset);
    PT_INIT(&pt_custom);
    PT_INIT(&pt_draw);
    
    //APinsPic
    mPORTASetPinsDigitalOut(BIT_4|BIT_2|BIT_1|BIT_0);    //Set port as output
    mPORTAClearBits(BIT_4|BIT_2|BIT_1|BIT_0);

    //BPinsPic
    mPORTBSetPinsDigitalOut(BIT_4|BIT_3|BIT_1|BIT_0);    //Set port as output
    mPORTBClearBits(BIT_4|BIT_3|BIT_1|BIT_0);

    //YPinsPE
    mPortYSetPinsIn(BIT_7|BIT_6|BIT_5|BIT_4|BIT_3);
    mPortYSetPinsOut(BIT_7|BIT_6|BIT_5|BIT_4|BIT_3);

    //ZPinsPE
    mPortZSetPinsIn(BIT_3|BIT_2|BIT_1|BIT_0);
    mPortZSetPinsOut(BIT_3|BIT_2|BIT_1|BIT_0);
    
    out_Y = 0;
    out_Z = 0;
    writePE(GPIOZ, out_Z);
    writePE(GPIOY, out_Y);
    pattern_A = 0; mPORTASetBits(pattern_A);
    pattern_B = 0; mPORTBSetBits(pattern_B);

    while (1){
      PT_SCHEDULE(protothread_bluetooth(&pt_bluetooth));
      PT_SCHEDULE(protothread_preset(&pt_preset));
      PT_SCHEDULE(protothread_custom(&pt_custom));
      PT_SCHEDULE(protothread_draw(&pt_draw));
    }
}//Main

// === end  ======================================================